@@ -93,6 +93,7 @@ meta_workspace_free (MetaWorkspace *workspace)
   
   g_free (workspace->work_areas);
 
+  g_list_free (workspace->mru_list);
   g_slist_free (workspace->left_struts);
   g_slist_free (workspace->right_struts);
   g_slist_free (workspace->top_struts);
@@ -111,11 +112,33 @@ meta_workspace_add_window (MetaWorkspace *workspace,
 {
   g_return_if_fail (!meta_workspace_contains_window (workspace, window));
   
+  /* If the window is on all workspaces, we want to add it to all mru
+   * lists, otherwise just add it to this workspaces mru list
+   */
+  if (window->on_all_workspaces) 
+    {
+      if (window->workspaces == NULL)
+        {
+          GList* tmp = window->screen->workspaces;
+          while (tmp)
+            {
+              MetaWorkspace* work = (MetaWorkspace*) tmp->data;
+              if (!g_list_find (work->mru_list, window))
+                work->mru_list = g_list_append (work->mru_list, window);
+
+              tmp = tmp->next;
+            }
+        }
+    }
+  else
+    {
+      g_assert (g_list_find (workspace->mru_list, window) == NULL);
+      workspace->mru_list = g_list_append (workspace->mru_list, window);
+    }
+
   workspace->windows = g_list_prepend (workspace->windows, window);
   window->workspaces = g_list_prepend (window->workspaces, workspace);
 
-  workspace->mru_list = g_list_append (workspace->mru_list, window);
-
   meta_window_set_current_workspace_hint (window);
   
   meta_window_queue_calc_showing (window);
@@ -141,7 +164,30 @@ meta_workspace_remove_window (MetaWorkspace *workspace,
 
   workspace->windows = g_list_remove (workspace->windows, window);
   window->workspaces = g_list_remove (window->workspaces, workspace);
-  workspace->mru_list = g_list_remove (workspace->mru_list, window);
+
+  /* If the window is on all workspaces, we don't want to remove it
+   * from the MRU list unless this causes it to be removed from all 
+   * workspaces
+   */
+  if (window->on_all_workspaces) 
+    {
+      if (window->workspaces == NULL)
+        {
+          GList* tmp = window->screen->workspaces;
+          while (tmp)
+            {
+              MetaWorkspace* work = (MetaWorkspace*) tmp->data;
+              work->mru_list = g_list_remove (work->mru_list, window);
+
+              tmp = tmp->next;
+            }
+        }
+    }
+  else
+    {
+      workspace->mru_list = g_list_remove (workspace->mru_list, window);
+      g_assert (g_list_find (workspace->mru_list, window) == NULL);
+    }
 
   meta_window_set_current_workspace_hint (window);
   
